# -*- coding: utf-8 -*-
"""
Created on Thu Jan 24 10:09:19 2019

@author: Gabriele Coiana
"""

import numpy as np

#def partition(omegas,T):
#    hbar = 1 #atomic units
#    kb = 3.1668085639379e-06 #atomic units
#    c = 0.0001519828500716 #Thz to Hartree
#    
#    
#    a = 1/(1-np.exp(-hbar*omegas/(kb*T)))
#    Z = np.prod(a)
#    return Z
#
#
#def free_energy2(omegas,Ts):
#    hbar = 1 #atomic units
#    kb = 3.1668085639379e-06 #atomic units
#    c = 0.0001519828500716 #Thz to Hartree
#    omegas = c*omegas*2*np.pi
#    F = []
#    for t in Ts:
#        print(np.log(partition(omegas,t)))
#        F.append(-kb*t*np.log(partition(omegas,t)))
#    F = np.array(F)
#    return F*627.509





















def bose_einstein(omegas,Ts):
    hbar = 1 #atomic units
    kb = 3.1668085639379e-06 #atomic units
    c = 0.0001519828500716 #Thz to Hartree
    omegas = c*omegas
    
    nk = 1/(np.exp(np.multiply(hbar*omegas,1/(kb*Ts.T))) - 1)
    n = np.sum(nk,axis=0)
    return n

def nk(omega,T):
    hbar = 1 #atomic units
    kb = 1#3.1668085639379e-06 #atomic units
    c = 0.16
    n = (np.exp(c*hbar*omega/(kb*T))-1)**(-1)
    return n


def free_energy(freqs,Ts):
    hbar = 1 #atomic units
    #kb = 3.1668085639379e-06 #atomic units
    c = 6.62607015*6.022*10/1000 #conversion frequency to kJ/mol
    
    F0 = []
    for temp in Ts:
        a = -kb*temp*(np.sum(np.log(nk(freqs,temp))))
        F0.append((0.5*np.sum(hbar*freqs)+a))
    F0 = np.array(F0)
#    n = bose_einstein(omegas,Ts)
#    F = 0.5*np.sum(hbar*omegas)-kb*np.multiply(Ts,np.log(n))
    return F0*c

def entropy(freqs,Ts,dT):
    F = free_energy(freqs,Ts) 
    S = -np.gradient(F,dT)
    return S*1000 #conversion to J/K/mol

def energy(freqs,Ts):
    hbar = 1 #atomic units
    #kb = 3.1668085639379e-06 #atomic units
    c = 6.62607015*6.022*10/1000 #conversion frequency to kJ/mol
    E0 = []
    for temp in Ts:
        #print(temp, nk(omegas,temp))
        #E0.append(float(np.real(np.dot(hbar*omegas.T,0.5+nk(omegas,temp)))))
        E0 = 0.5*np.sum(freqs) 
    E0 = np.array(E0)   
#    a = np.real((np.exp(hbar*np.dot(omegas,1/(kb*Ts.T)))-1)**(-1)+0.5)
#    E = np.dot(omegas.T,a).T
#    E = E.reshape(np.size(E))
    return E0*c

def cv(freqs,Ts,dT):
    E = energy(freqs,Ts)
    cv = np.gradient(E,dT)
    return cv*1000 #conversion to J/K/mol

## =============================================================================
## Parameters
#a = params.get_parameters()[0]
#mba, mti, mo = params.get_parameters()[1:4]
#N1,N2,N3 = params.get_parameters()[4:7]
#kinput = params.get_parameters()[7::][0]
#interp = params.get_parameters()[8]
#savefreq = params.get_parameters()[9]
#therm = params.get_parameters()[10]
#
#N1N2N3 = N1*N2*N3 # Number of cells
#N = N1*N2*N3*5    # Number of atoms
# =============================================================================
#omegas = np.array([1,2,3])
#omegas = omegas.reshape(len(omegas),1)
#Ts = np.arange(1,101)
#Ts = Ts.reshape(len(Ts),1)
#E = energy(omegas,Ts)
#print(E)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 12 14:03:17 2018

@author: Gabriele Coiana
"""

import numpy as np
import os

from phonon import bz
from phonon import cell



def K(N1N2N3):
    """
    Builds the stiffness matrix
    """
    N = N1N2N3*5
    K = np.zeros((15,N*3))
    with open('FORCE_SETS','r') as f:
        A = f.readlines()
    
    i = 0
    for l in range(5,(3+N)*15,3+N):
        B = A[l:l+N]
        with open('try','w') as g:
            g.writelines(B)
        C = np.loadtxt('try')
        D = C.reshape(1,N*3)
        K[i,:] = D
        i = i + 1
    os.remove('try')  
    return -K/(0.001*0.5292)

def R_matrix(N1,N2,N3,mode):
    """
    returns the R matrix
    """
    N1N2N3 = N1 * N2 * N3
    N = N1N2N3*5
        
    if(mode=='norelax' or mode=='cubic'):
        bati = cell.BaTiO3(4)
        bati.supercell(N1,N2,N3)
        Rba = bati.Ba.reshape(1,N1N2N3*3) 
        Rti = bati.Ti.reshape(1,N1N2N3*3) 
        Ro1 = bati.O1.reshape(1,N1N2N3*3) 
        Ro2 = bati.O2.reshape(1,N1N2N3*3) 
        Ro3 = bati.O3.reshape(1,N1N2N3*3)
        
        Ractual = np.hstack((Rba,Rti,Ro1,Ro2,Ro3))
        
        R0 = Ractual - np.tile(Ractual[0,0:0+3],N)
        R0 = np.repeat(R0,3,axis=0)
        R1 = Ractual - np.tile(Ractual[0,3*N1N2N3:3*N1N2N3+3],N)
        R1 = np.repeat(R1,3,axis=0)
        R2 = Ractual - np.tile(Ractual[0,3*N1N2N3*2:3*N1N2N3*2+3],N)
        R2 = np.repeat(R2,3,axis=0)
        R3 = Ractual - np.tile(Ractual[0,3*N1N2N3*3:3*N1N2N3*3+3],N)
        R3 = np.repeat(R3,3,axis=0)
        R4 = Ractual - np.tile(Ractual[0,3*N1N2N3*4:3*N1N2N3*4+3],N)
        R4 = np.repeat(R4,3,axis=0)
        R = np.vstack((R0,R1,R2,R3,R4))
        
    else:
        Ractual = np.loadtxt('../../Relax_Supercell/Cartesian_M0',skiprows=2+N,usecols=(1,2,3)).reshape(1,N*3)
        R0 = Ractual - np.tile(Ractual[0,0:0+3],N)
        R0 = np.repeat(R0,3,axis=0)
        R1 = Ractual - np.tile(Ractual[0,3*N1N2N3:3*N1N2N3+3],N)
        R1 = np.repeat(R1,3,axis=0)
        R2 = Ractual - np.tile(Ractual[0,3*N1N2N3*2:3*N1N2N3*2+3],N)
        R2 = np.repeat(R2,3,axis=0)
        R3 = Ractual - np.tile(Ractual[0,3*N1N2N3*3:3*N1N2N3*3+3],N)
        R3 = np.repeat(R3,3,axis=0)
        R4 = Ractual - np.tile(Ractual[0,3*N1N2N3*4:3*N1N2N3*4+3],N)
        R4 = np.repeat(R4,3,axis=0)
        R = np.vstack((R0,R1,R2,R3,R4))
        
    return R#*0.529177249







def D(k, K,N1,N2,N3,mba, mti, mo, mode):
    """
    builds the dynamical matrix
    """
    N1N2N3 = N1 * N2 * N3
    N = N1N2N3*5
    m = np.array([mba,mba,mba,mti,mti,mti,mo,mo,mo,mo,mo,mo,mo,mo,mo])
           
    #prepare kdotr
    R = R_matrix(N1,N2,N3,mode)
    kdotr = np.zeros((15,N*3))
    for i in range(15):
        for j in range(0,N*3,3):
            kdotr[i,j] = np.dot(k,R[i,j:j+3])
            kdotr[i,j+1] = np.dot(k,R[i,j:j+3])
            kdotr[i,j+2] = np.dot(k,R[i,j:j+3])
    
    D = np.zeros((15,15),dtype=complex)
    for i in range(15):
        for j,h in zip(range(15),np.repeat(range(0,N),3)):
            mass_coeff = complex(1/(m[i]*m[j])**(0.5))
            exp = 0*1j
            #h = 0
            for l in range(j+h*3*(N1N2N3-1),j+h*3*(N1N2N3-1)+(N1N2N3)*3,3):
                #print(l)
                #print(np.exp(1j*kdotr[i,l]),K[i,l])
                exp = exp + K[i,l]*np.exp(1j*kdotr[i,l])
            #print()
            D[i,j] = mass_coeff*exp
    return D*0.964*10**(4)#, kdotr, R



## =============================================================================
## Parameters
#a = params.get_parameters()[0]
#mba, mti, mo = params.get_parameters()[1:4]
#N1,N2,N3 = params.get_parameters()[4:7]
#kinput = params.get_parameters()[7::][0]
#interp = params.get_parameters()[8]
#savefreq = params.get_parameters()[9]
#therm = params.get_parameters()[10]
#
#N1N2N3 = N1*N2*N3 # Number of cells
#N = N1*N2*N3*5    # Number of atoms
# =============================================================================













#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Nov 13 18:35:54 2018

@author: Gabriele Coiana
"""
import numpy as np

from phonon import params

def getk(a,N1,N2,N3,kinput_scaled):
    # =============================================================================
    #   Brilloin zone for FCC
    c = 2*np.pi/a
    b1 = c*np.array([-1,1,1])/np.sqrt(3)
    b2 = c*np.array([1,-1,1])/np.sqrt(3)
    b3 = c*np.array([1,1,-1])/np.sqrt(3)
    b = np.linalg.norm(b1)
    kinput_real = np.array(kinput_scaled)*b
    #Compute all possible k points
    x1 = np.arange(0,N1+1)/N1
    x2 = np.arange(0,N2+1)/N2
    x3 = np.arange(0,N3+1)/N3
    comb = np.array(np.meshgrid(x1,x2,x3)).T.reshape(-1,3)
    #a1 = np.multiply(comb,b1)
    #a2 = np.multiply(comb,b2)
    #a3 = np.multiply(comb,b3)
    #kpoints = a1+a2+a3
    allkpoints_scaled = comb
    allkpoints = comb*b
    # =============================================================================
    
    
    # =============================================================================
    # Compute directions <x y z>
    directions_scaled = []
    directions = []
    for i in range(0,len(kinput_scaled)-1):
        if(kinput_scaled[i] not in allkpoints_scaled):
            print('The k point '+str(kinput_scaled[i])+' provided is not compatible with the supercell. \nTry again :)')
        num = 1/np.max(np.abs(kinput_scaled[i]-kinput_scaled[i+1]))
        mod = np.linalg.norm(kinput_scaled[i]-kinput_scaled[i+1])
        directions_scaled.append((kinput_scaled[i]-kinput_scaled[i+1])*num)
        directions.append((kinput_scaled[i]-kinput_scaled[i+1])/mod)
    # =============================================================================
    
    # =============================================================================
    # Remove kpoints already input
    indexes = []
    for i in range(len(allkpoints_scaled)):
        for kpoint in kinput_scaled:
            if(np.array_equal(allkpoints_scaled[i],kpoint)):
                indexes.append(i)
    allkpoints_scaled_no = np.delete(allkpoints_scaled,indexes,axis=0)
    # =============================================================================
    
    # =============================================================================
    # Look for other k points in the same path
    kdef = kinput_real
    kdef_scaled = kinput_scaled
    count = 0
    for index in range(0,len(directions)): #iterating over directions
        count = count+1
        direction = directions[index]
        
        if (all(direction <= np.array([0,0,0]))):
            for kpoint in allkpoints_scaled_no:
                diff = kinput_scaled[index]-kpoint
                mod = np.linalg.norm(diff)
                actual_direction = diff/mod
                if(np.array_equal(np.round(actual_direction,9),np.round(direction,9)) and all(np.abs(diff)<=np.abs(kinput_scaled[index+1]-kinput_scaled[index]))):
                    kdef = np.insert(kdef,count,kpoint*b,axis=0)
                    kdef_scaled = np.insert(kdef_scaled,count,kpoint,axis=0)
                    count = count+1 
        else:
            for kpoint in allkpoints_scaled_no[::-1]:
                diff = kinput_scaled[index]-kpoint
                mod = np.linalg.norm(diff)
                actual_direction = diff/mod
                if(np.array_equal(np.round(actual_direction,9),np.round(direction,9)) and all(np.abs(diff)<=np.abs(kinput_scaled[index+1]-kinput_scaled[index]))):
                    kdef = np.insert(kdef,count,kpoint*b,axis=0)
                    kdef_scaled = np.insert(kdef_scaled,count,kpoint,axis=0)
                    count = count+1 
            
    # =============================================================================
    
    # =============================================================================
    #  fake k points only for plotting, projection
    directionsdef_scaled = []
    for i in range(0,len(kdef_scaled)-1):
        directionsdef_scaled.append((kdef_scaled[i]-kdef_scaled[i+1]))
    kk = np.zeros(len(kdef_scaled))
    for i in range(1,len(kk)):
        kk[i] = kk[i-1]+(np.linalg.norm(directionsdef_scaled[i-1]))
    # =============================================================================
           
    return kdef, kdef_scaled, kk, allkpoints








#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 12 18:21:44 2018

@author: Gabriele Coiana
"""
import numpy as np

import sys,os
#sys.path.append(os.getcwd())

from phonon import params
from phonon import dynam
from phonon import plot
from phonon import bz
from phonon import cell
from phonon import thermal

# =============================================================================
# Parameters
a = params.get_parameters()[0]
mba, mti, mo = params.get_parameters()[1:4]
N1,N2,N3 = params.get_parameters()[4:7]
kinput = params.get_parameters()[7::][0]
interp = params.get_parameters()[8]
savefreq = params.get_parameters()[9]
therm = params.get_parameters()[10]

N1N2N3 = N1*N2*N3 # Number of cells
N = N1*N2*N3*5    # Number of atoms
# =============================================================================

#if (__name__ == "__main__"):
print('\nPho-NON-py')
print()
print('\nCalculating the k points...')
ks, ks_scaled, kk, allks = bz.getk(a,N1,N2,N3,kinput)
print('Ok babe, Im gonna take all of these kpoints (scaled):\n',ks_scaled)


print('\nComputing the stiffness matrix...')
K = dynam.K(N1N2N3)
print('\tK is equilibrated to within ',np.max(np.abs(np.sum(K,axis=1))))

print('\nCalculating dynamical matrixes...')
FREQ = []
mode = str(sys.argv[1])
for kpoint in ks:
    D = dynam.D(kpoint,K,N1,N2,N3,mba,mti,mo,mode)
    
    ll = np.matrix(D)
    print('\t',str(kpoint),' Hermitian: ',np.allclose(ll,ll.getH(),rtol=0.01,atol=0.01))
    
    eigvals, eigvec = np.linalg.eig(D)
    idx = eigvals.argsort()[::-1]   
    eigvals = eigvals[idx]  
    eigvec = eigvec[:,idx]
    freq = (np.sqrt(eigvals)/(2*np.pi))
    FREQ.append(freq)
print()


FREQ = np.array(FREQ).T

if(savefreq):
    np.savetxt('frequencies',np.column_stack([FREQ.real, FREQ.imag]))
    np.savetxt('frequencies_niceview',FREQ, fmt='%-1.2d')



plot.plot(ks_scaled,kk, FREQ,interp)


if (therm):
    ks, ks_scaled, kk, allks = bz.getk(a,N1,N2,N3,kinput)
    FREQ = []
    print(np.shape(allks))
    for kpoint in allks:
        print(kpoint)
        D = dynam.D(kpoint,K,N1,N2,N3,mba,mti,mo,mode)
        
        eigvals, eigvec = np.linalg.eig(D)
        idx = eigvals.argsort()[::-1]   
        eigvals = eigvals[idx]  
        eigvec = eigvec[:,idx]
        freq = (np.sqrt(eigvals)/(2*np.pi))
        FREQ.append(freq)
    FREQ = np.array(FREQ)
    
    freq = FREQ.reshape((np.size(FREQ),1))
    
    c = 6.626*6.022*1e+01
    c1 = 0.3990
    #print(.5*np.sum(freq)*c1/N)
    
    dT = 10
    T = np.arange(10,1000,dT)
    
    F = thermal.free_energy(freq,T)/N
    S = thermal.entropy(freq,T,dT)
    
    dulong_petite = 3*N*1.38064852*1e-23
    T = T.reshape(len(T),1)
    E = thermal.energy(freq,T)/N
    print(E)
    
    
    
    
    
    cv = thermal.cv(freq,T,dT)
    
    #plot.plot_thermal(T,E,F,S,cv)
    
    
























#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Feb 21 16:09:35 2019

@author: Gabriele Coiana
"""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="phonon0K",
    version="0.0.2",
    author="Gabriele Coiana",
    author_email="gabriele.coiana17@imperial.ac.uk",
    description="Harmonic phonon package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/Coiana/phonon0K",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    scripts=['bin/phonon0K'],
)
